// main.js

// Modules to control application life and create native browser window
const { app, BrowserWindow, ipcMain, Menu } = require('electron')
const path = require('path')
// const fs = require('fs')
// const url = require('url');
const { autoUpdater } = require('electron-updater');
const feedUrl = `http://www.xiaoqiumi.com/xqmTerminal/${process.platform}`;
const pkg = require(path.join(__dirname, '../package.json'))
  

console.log("I'm coming!");

const mainURL = "https://www.xiaoqiumi.com/index.html#/SignIn";
// const mainURL = "http://192.168.0.182:8080/index.html#/SignIn";
// const mainURL = "http://192.168.0.182:8080/livetemp.html#/?matchid=400003774&sporttype=1&mode=liveDP";

const gotTheLock = app.requestSingleInstanceLock();
if(!gotTheLock){
  app.quit()
  return
}

//设置默认菜单
let menu = Menu.buildFromTemplate( [
  {
    label: '工具',
    submenu: [
      {
        label: '退出',
        role: 'quit'
      },
      { type: 'separator' },
    ]
  },
  {
    label: '视图',
    submenu: [
      {
        label: '重新加载',
        role: 'reload'
      },
      {
        label: '清除缓存加载',
        role: 'forcereload'
      },
      {
        label: '全屏切换',
        role: 'togglefullscreen'
      },
    ]
  }
]);
Menu.setApplicationMenu(menu);


var mainWindow = null;
var windowList = {};
const createWindow = (mainURL, fullscreen) => {
  // Create the browser window.
  var win = new BrowserWindow({
    width: fullscreen ? 1920 : 1280,
    height: fullscreen ? 1080 : 720,
    fullscreenWindowTitle: false,
    title: pkg.fullName,
    // fullscreen: true,
    // autoHideMenuBar: true,
    icon: path.join(__dirname, 'build/logo.png'),
    webPreferences: {
      contextIsolation: false,
      preload: path.join(__dirname, 'preload.js')
    }
  })
  // win.webContents.openDevTools();
  if(!mainWindow) mainWindow = win;
  // win.maximize();
  
  win.addListener('enter-full-screen', function(){
    win.menuBarVisible = false;
  })

  win.addListener('leave-full-screen', function(){
    win.menuBarVisible = true;
  })

  ipcMain.on('routerChange', myRouterChange)

  win.on('closed', function (e) {
    console.log("win.on('closed'", e);
    ipcMain.removeListener('routerChange', myRouterChange)
  })
  
  
  function myRouterChange (e, arg) {
    if (arg) {
      if (arg === 'back') {
        win && win.webContents.goBack()
      } else {
        win && win.goForward()
      }
    }
  }
  
  // win.loadFile('startup/testmove.html', {userAgent: 'xqmTerminal'});
  // win.loadFile('dist/index.html');
  // win.loadURL('https://www.xiaoqiumi.com/index.html#/SignIn', {userAgent: 'xqmTerminal'});
  windowList[win.id] = win;
  win.loadURL(mainURL, {userAgent: `xqmTerminal, windowID=${win.id}`});
  // 注册新窗口事件，以处理原有的window.open
  win.webContents.setWindowOpenHandler((e)=>{
    createWindow(e.url);
    return { action: 'deny' };
  });
  
}

const createSubWindow = ({url, width=1280, height=200,frame=false})=>{
  const subWindow = new BrowserWindow({
    width: width,
    height: height,
    backgroundColor: "#00ff00",
    icon: path.join(__dirname, 'build/logo.png'),
    frame,
    hasShadow: false,
    transparent: true,
    titleBarStyle: 'customButtonsOnHover',
    minimizable: false,
    maximizable: false,
    resizable: false,
    webPreferences: {
      // nodeIntegration: true,
      backgroundThrottling: false,
      contextIsolation: false,
      preload: path.join(__dirname, 'preload.js')
    }
  });
  // console.log("subWindow", subWindow, subWindow.webContents.getBackgroundThrottling());
  // subWindow.webContents.setBackgroundThrottling(false);
  // console.log("subWindow2", subWindow, subWindow.webContents.getBackgroundThrottling());
  
  // subWindow.webContents.openDevTools();
  
  windowList[subWindow.id] = subWindow;
  subWindow.on('closed', (e)=> console.log("subWindow.on('closed')", e));
  subWindow.loadURL(url, {userAgent: `xqmTerminal, windowID=${subWindow.id}`});
  subWindow.show();
  // subWindow.setAlwaysOnTop(true, 'floating', 1000);
}


//自动更新
let sendUpdateMessage = (message, data) => {
    console.log(message, data);
    mainWindow.webContents.send('message', { message, data });
};
let checkForUpdates = () => {
    autoUpdater.setFeedURL(feedUrl);
    autoUpdater.on('error', function (message) {
        sendUpdateMessage('error', message)
    });
    autoUpdater.on('checking-for-update', function (message) {
        sendUpdateMessage('checking-for-update', message)
    });
    autoUpdater.on('update-available', function (message) {
        sendUpdateMessage('update-available', message)
    });
    autoUpdater.on('update-not-available', function (message) {
        sendUpdateMessage('update-not-available', message)
    });
    // 更新下载进度事件
    autoUpdater.on('download-progress', function (progressObj) {
        sendUpdateMessage('downloadProgress', progressObj)
    })
    autoUpdater.on('update-downloaded', function (event, releaseNotes, releaseName, releaseDate, updateUrl, quitAndUpdate) {
        serv.appNewVersion = releaseName;
        sendUpdateMessage('isUpdateNow', {releaseName});
    });
    //执行自动更新检查
    autoUpdater.checkForUpdates();
};
ipcMain.on('updateNow',(event, setting)=> {
  if(serv.appNewVersion && serv.appNewVersion != serv.appVersion)
    autoUpdater.quitAndInstall();
})


app.commandLine.appendSwitch('disable-background-timer-throttling');
// 这段程序将会在 Electron 结束初始化
// 和创建浏览器窗口的时候调用
// 部分 API 在 ready 事件触发后才能使用。
app.whenReady().then(() => {
  createWindow(mainURL)
  // setTimeout(checkForUpdates, 5000);

  app.on('activate', () => {
    // On macOS it's common to re-create a window in the app when the
    // dock icon is clicked and there are no other windows open.
    if (BrowserWindow.getAllWindows().length === 0) createWindow(mainURL)
  })
  
  ipcMain.on('openWindow', function(event, arg){
    createWindow(arg, true);
  })

  console.log("register openSubTitle");
  // 创建字幕
  ipcMain.on('openSubTitle', function(e, arg){
    console.log("openSubTitle", arg);
    createSubWindow(arg);
  })
  
  ipcMain.on('hide-window', function(event, arg){
    var win = windowList[arg];
    if(win) win.minimize();
  })
  
})

// 除了 macOS 外，当所有窗口都被关闭的时候退出程序。 There, it's common
// for applications and their menu bar to stay active until the user quits
// explicitly with Cmd + Q.
app.on('window-all-closed', () => {
  if (process.platform !== 'darwin') app.quit()
})

// In this file you can include the rest of your app's specific main process
// code. 也可以拆分成几个文件，然后用 require 导入。

